# IEEE Common Format codes:
#   bus_type = 0  PQ load bus
#   bus_type = 2  PV generation bus
#   bus_type = 3  slack/reference bus
######################################################################

######################################################################
#  DEFINE THE AMPL MODEL FOR OPF
######################################################################

reset;

#---- DECLARE SPACE FOR UP TO 4000 BUSES.
set BUS;
set BRANCH within {1..4000} cross BUS cross BUS;


set Time;

#---- DECLARE ITEMS TO BE READ FROM THE .bus DATA FILE.
param bus_type       {BUS};
param bus_name       {BUS} symbolic;

param bus_p_gen_max  {BUS};
param bus_Gen_ramp   {BUS};
param SocMax		 {BUS};
param PchargeMax     {BUS};
param PchargeRate    {BUS};
param PCDC0          {BUS};
param PG0            {BUS};
param SocInit		 {BUS};


param bus_p_load_test     {BUS,Time};
param bus_gen_cost        {BUS,Time};
param bus_p_inj_set       {BUS,Time};


#---- DECLARE ITEMS TO BE READ FROM THE .branch DATA FILE.
param line_limit     {BRANCH};

#---- DECLARE VARIABLES, WITH UPPER AND LOWER BOUNDS.

var bus_angle   {i in BUS , Time };
var Pcharge {BUS , Time};
var Pdcharge {BUS , Time};
var SOC		{BUS ,Time};


#---- DEFINE A SET FOR THE BUS ADMITTANCE MATRIX (YBUS).
set YBUS := setof{i in BUS} (i,i) union 
              setof {(l,k,m) in BRANCH} (k,m) union
                setof {(l,k,m) in BRANCH} (m,k);

#---- COMPUTE ADMITTANCE VALUES FROM IEEE COMMON FORMAT DATA.
#param G {BUS,BUS};
param B {BUS,BUS};




var bus_p_inj {k in BUS,p in Time : bus_type[k] == 2 || bus_type[k] == 3} =
    (Pcharge[k,p]-Pdcharge[k,p]) + bus_p_load_test[k,p] + sum{(k,m) in YBUS} ( 
                     
                    + B[k,m] * (bus_angle[k,p] - bus_angle[m,p]));

var line_flow {(l,k,m) in BRANCH , p in Time} =
 B[k,m]*(bus_angle[k,p] - bus_angle[m,p]);
 
 

#---- OBJECTIVE FUNCTION.

minimize  active_power : 
  sum{p in Time, k in BUS : bus_type[k] == 2 || bus_type[k] == 3} 
  bus_gen_cost[k,p]*(bus_p_inj[k,p]-bus_p_inj_set[k,p])^2 ;



#---- AC POWER FLOW BALANCE EQUATIONS (NONLINEAR CONSTRAINTS).


subject to p_load { k in BUS , p in Time : bus_type[k] == 1}:
  -(Pcharge[k,p]-Pdcharge[k,p]) - bus_p_load_test[k,p]
  - sum{(k,m) in YBUS } (
                         
                + B[k,m] * (bus_angle[k,p] - bus_angle[m,p])) = 0;



 subject to generator_ramp_limit { k in BUS , p in Time :  (bus_type[k] == 2 || bus_type[k] == 3) && p>1  }:

-bus_Gen_ramp[k] <= bus_p_inj[k,p]-bus_p_inj[k,p-1] <= bus_Gen_ramp[k];

 subject to generator_ramp_limit0 { k in BUS :  (bus_type[k] == 2 || bus_type[k] == 3) }:

-1*bus_Gen_ramp[k] <= bus_p_inj[k,1]-PG0[k] <= 1*bus_Gen_ramp[k];


subject to p_inj {k in BUS , p in Time : bus_type[k] == 2 || bus_type[k] ==3}:
   0 <= bus_p_inj[k,p]  <= bus_p_gen_max[k] ;
   
   subject to line_limit_constraint { (l,k,m) in BRANCH , p in Time}:

  -line_limit[l,k,m] <= line_flow[l,k,m,p] <= line_limit[l,k,m]; 
  
subject to PchargeLim { k in BUS , p in Time}:
 0 <= Pcharge[k,p] <= PchargeMax[k] ;
 
 
  subject to PdchargeLim { k in BUS , p in Time}:
 0 <= Pdcharge[k,p] <= PchargeMax[k] ;
 #-0.2*SocMax[k] <= Pcharge[k,p] <= 0.2*SocMax[k] ;
 
 subject to SOCrate { k in BUS, p in Time : p>1}:
 SOC[k,p] = SOC[k,p-1]+(1/60)*(0.99*Pcharge[k,p]-1.01*Pdcharge[k,p]);
 
 subject to SOCLim { k in BUS , p in Time}:
 0 <= SOC[k,p] <= SocMax[k];
 
 subject to SCO_init { k in BUS }:
 SOC[k,1] = SocInit[k]+(1/60)*(0.99*Pcharge[k,1]-1.01*Pdcharge[k,1]);
 
 subject to CDCRate { k  in BUS, p in Time : p>1}:
  -PchargeRate[k] <=  (Pcharge[k,p]-Pdcharge[k,p])-(Pcharge[k,p-1]-Pdcharge[k,p-1]) <= PchargeRate[k];
  
 subject to PchargeRate_init { k in BUS}:
  -PchargeRate[k] <=  (Pcharge[k,1]-Pdcharge[k,1])- PCDC0[k] <= PchargeRate[k];
 #subject to sustain { k in BUS  }:
 #SOC[k,5] = SocInit[k];

